package Mail;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

public class Connection {
    private static final byte[] CRLF = new byte[] {13, 10};
    private boolean debug,
					connected;
    private byte[] buffer = new byte[128];
    private int count,
				position;
	private StreamConnection connection;
	private DataInputStream in;
	private DataOutputStream out;
	
	public int 	bytesIn = 0,
				bytesOut = 0;
	
    public Connection(){
    }
    
    public void open(String host, int port, boolean ssl) throws IOException {
        String protocol = (ssl ? "ssl" : "socket");
    	connection = (StreamConnection)Connector.open (protocol + "://" + host + ":" + port, Connector.READ_WRITE, true);
        in = connection.openDataInputStream();
        out = connection.openDataOutputStream();
        connected = true;
	}
    public void close() throws IOException {
        connected = false;
        //   .
        if (in != null)
        	try {in.close();} catch (IOException ex){}
        if (out != null)
        	try {out.close();} catch (IOException ex){}
        if (connection != null)
        	try {connection.close();} catch (IOException ex){}
        	
        in = null;
        out = null;
        connection = null;
	}
    
    protected void write(byte[] buffer, int offset, int count) throws IOException {
    	bytesOut += count;
    	out.write(buffer, offset, count);
	}
    protected int read(byte[] buffer, int offset, int count) throws IOException {
    	bytesIn += count;
		return in.read(buffer, offset, count);
	}
    public boolean connected() {
		return connected;
	}

    public void send(String s) throws IOException {
        byte[] bytes = s.getBytes();
        int length = bytes.length;

        if (s.length() == 0) {
            if (debug) System.out.println("[SEND]");

            write(CRLF, 0, 2);
        }else {
            int i = 0;
            while (i < length) {
                int j = i;

                while ((j < length) && (bytes[j] != 0x0A) && (bytes[j] != 0x0D)) {
                    j++;
                }

                if (debug) System.out.println("[SEND] " + s.substring(i, j));

                write(bytes, i, j - i);
                write(CRLF, 0, 2);

                if ((j < length - 1) && (bytes[j] == 0x0D) && (bytes[j + 1] == 0x0A)) {
                    j++;
                }

                i = j + 1;
            }
        }
    }

    public String receive() throws IOException {
        String result = "";
        boolean stop = false;

        while (!stop) {
            count = 0;

	        while (true) {
                int actual = read(buffer, count, 1);
                
	            if (actual == -1) {
	                try {
	                    close();
	                } catch (IOException ignored) {}
	                
	                throw new IOException("Connection closed");
	            }
                
	            else if (actual == 0) {
	                try {
	                    Thread.yield();
	                } catch (Exception ignored) {}
	            }
                
	            else {
                    byte b = buffer[count];
                    
                    if (b == 0x0D) {
                        /* Ignore CRs */
                    }
                    else if (b == 0x0A) {
                        stop = true;
                        break;
                    }
                    else {
                        count++;
                        if (count == buffer.length) break;
                    }
	            }
	        }

            result = result + new String(buffer, 0, count);
        }
        
        if (debug) System.out.println("[RECV] " + result);

        return result;
    }

    public void setDebug(boolean debug) {
        this.debug = debug;
    }

    public boolean getDebug() {
        return debug;
    }
}