package Mail;

import java.io.IOException;

import lib.Utils.Utils;

public class ImapClient extends InboxClient {

    private int commandCount = 0;

    public ImapClient() {
        super(null);
    }

    public ImapClient(Connection connection) {
        super(connection);
    }

    public void open(String host, int port, String user, String pass, boolean ssl) throws IOException, MailException {
        if (connected()) close();

        connection.open(host, (port == 0 ? 143 : port), ssl);

        try {
            execute("LOGIN", user + " " + pass, null);
            execute("SELECT", "INBOX", null);
        }
        catch (MailException e) {
            connection.close();

            throw e;
        }
    }

    public void close() throws IOException, MailException {
        if (connected()) {
            execute("CLOSE", null, null);
            execute("LOGOUT", null, null);
        }

        connection.close();
    }

    private String execute(String command, String arguments, Message message) throws IOException, MailException {
        String result = null;

        String tag = "A" + commandCount++ + " ";
        connection.send(tag + command + (arguments == null ? "" : " " + arguments));

        String temp = connection.receive();
        while (!temp.startsWith(tag)) {
            if (temp.indexOf(" " + command + " ") != -1) {
                int p = temp.indexOf('(');
                int q = temp.indexOf(')', p + 1);

                if (p != -1) {
                    if (q > p) {
                        result = temp.substring(p + 1, q);
                    }
                    else if (message != null) {
                        int left = temp.indexOf('{');
                        int right = temp.indexOf('}', left);

                        receiveMessage(message, Integer.parseInt(temp.substring(left + 1, right)));
                    }
                }
            }
            temp = connection.receive();
        }

        temp = temp.substring(tag.length());
        if (temp.startsWith("BAD ") || temp.startsWith("NO ")) {
            throw new MailException(temp);
        }

        return result;
    }

    public int getMessageCount() throws IOException, MailException {
        String buffer = execute("STATUS", "INBOX (MESSAGES)", null);

        int space = buffer.indexOf(' ');

        return Integer.parseInt(buffer.substring(space + 1));
    }

    private void receiveMessage(Message message, int size) throws IOException, MailException {
    	message.size = Utils.getKBytes(size)+" ";
        int count = 0;

        String buffer = connection.receive();
        int octets = buffer.length() + 2;

        while (!(buffer.equals(""))) {
            if (buffer.startsWith(" ") || buffer.startsWith("\t")) {
                message.setHeaderLine(count - 1, message.getHeaderLine(count - 1) + "\r\n" + buffer);
            } else {
                message.addHeaderLine(buffer);
                count++;
            }

            buffer = connection.receive();
            octets = octets + buffer.length() + 2;
        }

        while (octets < size) {
            buffer = connection.receive();
            octets = octets + buffer.length() + 2;
            message.addBodyLine(buffer);
        }
    }

    public Message getMessage(int index) throws IOException, MailException {
        Message message = new Message();
        execute("FETCH", (index + 1)+ " (RFC822)", message);
        return message;
    }

    public Message getHeaders(int index) throws IOException, MailException {
        Message message = new Message();
        execute("FETCH", (index + 1)+ " (RFC822.HEADER)", message);
        return message;
    }

    public void removeMessage(int index) throws IOException, MailException {
        execute("STORE", (index + 1) + " +FLAGS.SILENT (\\DELETED)", null);
    }

    public String getUniqueId(int index) throws IOException, MailException {
        String buffer = execute("FETCH", (index + 1) + " (UID)", null); // in Klammern?

        int space = buffer.indexOf(' ');

        return buffer.substring(space + 1);
    }

    public int getSize(int index) throws IOException, MailException {
        String buffer = execute("FETCH", (index + 1) + " (RFC822.SIZE)", null); // in Klammern?

        int space = buffer.indexOf(' ');

        return Integer.parseInt(buffer.substring(space + 1));
    }

    //==========  ==============
    public int getBytesIn(){
    	return connection.bytesIn;
    }
    public int getBytesOut(){
    	return connection.bytesOut;
    }
}
