package Mail;

import java.util.Calendar;
import java.util.Random;
import java.util.TimeZone;
import java.util.Vector;

public class Message {

    private Vector lines = new Vector();

    private int headerSize = 0;
    
    public String size = "0 ";

    public Message() {
        lines.addElement("");
    }

    public Message(String from, String to, String subject) {
        lines.addElement("");

        if (from != null) {
            addHeaderLine("From: " + from);

            String s = getMachineAddress(from);
            addHeaderLine("Message-ID: <" + getRandomString() + "." + s + ">");
        } else {
            addHeaderLine("Message-ID: <" + getRandomString() + ">");
        }

        if (to != null) {
            addHeaderLine("To: " + to);
        }

        if (subject != null) {
            addHeaderLine("Subject: " + subject);
        }

        addHeaderLine("Date: " + getCanonicalDate(Calendar.getInstance(), TimeZone.getDefault()));
    }

    public String getHeaderLine(int index) throws ArrayIndexOutOfBoundsException {
        if ((index < 0) || (index >= headerSize)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        return (String)lines.elementAt(index);
    }

    public String getHeaderName(int index) throws ArrayIndexOutOfBoundsException {
        return getStringName(getHeaderLine(index));
    }

    public String getHeaderValue(int index) throws ArrayIndexOutOfBoundsException {
        return getStringValue(getHeaderLine(index));
    }

    public int getHeaderIndex(String name, int startIndex) {
        String lowerName = name.toLowerCase();

        for (int i = startIndex; i < headerSize; i++) {
            String s = getHeaderName(i);
            if ((s != null) && (s.toLowerCase().equals(lowerName))) {
                return i;
            }
        }

        return -1;
    }

    public int getHeaderIndex(String name) {
        return getHeaderIndex(name, 0);
    }

    public String getHeaderValue(String name) {
        return getHeaderValue(name, null);
    }

    public String getHeaderValue(String name, String def) {
        int i = getHeaderIndex(name);

        if (i == -1) {
            return def;
        }
        else {
            return getHeaderValue(i);
        }
    }

    public String[] getAllHeaderValues(String name) {
        Vector lines = new Vector();

        int i = getHeaderIndex(name);
        while (i != -1) {
            lines.addElement(getHeaderValue(i));
            i = getHeaderIndex(name, i + 1);
        }

        String[] result = new String[lines.size()];
        for (int j = 0; j < lines.size(); j++) {
            result[j] = (String)lines.elementAt(j);
        }

        return result;
    }

    public void setHeaderLine(int index, String line) {
        if ((index < 0) || (index >= headerSize)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.setElementAt(line, index);
    }

    public int getHeaderLineCount() {
        return headerSize;
    }

    public int addHeaderLine(String line) {
        lines.insertElementAt(line, headerSize);
        return headerSize++;
    }

    public void insertHeaderLine(int index, String line) {
        if ((index < 0) || (index > headerSize)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.insertElementAt(line, index);
        headerSize++;
    }

    public void removeHeaderLine(int index) {
        if ((index < 0) || (index >= headerSize)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.removeElementAt(index);
        headerSize--;
    }

    public void setHeaderValue(String name, String value) {
        int i = getHeaderIndex(name);
        if (i == -1) {
            if (value != null) {
                addHeaderLine(name + ": " + value);
            }
        }
        else {
            if (value != null) {
                setHeaderLine(i, name + ": " + value);
            }
            else {
                removeHeaderLine(i);
            }
        }
    }

    public String getBodyLine(int index) {
        if ((index < 0) || (index >= getBodyLineCount())) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        return (String)lines.elementAt(headerSize + index + 1);
    }

    public void setBodyLine(int index, String line) {
        if ((index < 0) || (index >= getBodyLineCount())) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.setElementAt(line, headerSize + index + 1);
    }

    public int getBodyLineCount() {
        return lines.size() - headerSize - 1;
    }

    public int addBodyLine(String line) {
        lines.addElement(line);
        return getBodyLineCount() - 1;
    }

    public void insertBodyLine(int index, String line) {
        if ((index < 0) || (index > getBodyLineCount())) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.insertElementAt(line, headerSize + index + 1);
    }

    public void removeBodyLine(int index) {
        if ((index < 0) || (index >= getBodyLineCount())) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        lines.removeElementAt(headerSize + index + 1);
    }

    public static String getMachineAddress(String address) {
        int p, q;

        p = address.indexOf('<');
        q =  address.indexOf('>', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(p + 1, q);

        p = address.indexOf('(');
        q = address.indexOf(')', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(0, p).trim();

        p = address.indexOf('"');
        q = address.indexOf('"', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(0, p).trim();

        return address;
    }

    public static String getDisplayAddress(String address) {
        int p, q;

        p = address.indexOf('"');
        q = address.indexOf('"', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(p + 1, q);

        p = address.indexOf('(');
        q = address.indexOf(')', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(p + 1, q);

        p = address.indexOf('<');
        q =  address.indexOf('>', p + 1);
        if ((p != -1) && (q != -1)) return address.substring(0, p).trim();

        return "";
    }

    /**
     * Returns a canonical representation of an e-mail address. It basically
     * consists of the username and domain parts enclosed in angular brackets.
     * It does not contain the user's realname.
     * [May be removed in favour of getMachineAddress or expanded to hold
     * display *and* machine address.]
     */
        /*
        public static String getCanonicalAddress(String address) {
                int p = address.indexOf('<');
                int q =  address.indexOf('>', p + 1);
                if ((p != -1) && (q != -1)) return address.substring(p, q + 1);

                p = address.indexOf('(');
                q = address.indexOf(')', p + 1);
                if ((p != -1) && (q != -1)) return "<" + address.substring(0, p) + ">";

                p = address.indexOf('"');
                q = address.indexOf('"', p + 1);
                if ((p != -1) && (q != -1)) return "<" + address.substring(0, p) + ">";

                return "<" + address + ">";
        }
         */

    private static String intToStr(int value, int length) {
        String result = Integer.toString(value);
        result = "0000".substring(result.length(), length) + result;
        return result;
    }

    public static String getCanonicalDate(Calendar calendar, TimeZone timezone) {
        String monthNames = "JanFebMarAprMayJunJulAugSepOctNovDec";
        String dayNames = "SunMonTueWedThuFriSatSun";

        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        int weekday = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        int minute = calendar.get(Calendar.MINUTE);
        int second = calendar.get(Calendar.SECOND);

        String result = dayNames.substring(3 * weekday, 3 * weekday + 3) + ", "
        + intToStr(day, 2) + " "
        + monthNames.substring(3 * month, 3 * month + 3) + " "
        + intToStr(year, 4) + " "
        + intToStr(hour, 2) + ":"
        + intToStr(minute, 2) + ":"
        + intToStr(second, 2);

        if (timezone != null) {
            int offset = timezone.getRawOffset() / 1000;
            if (timezone.useDaylightTime()) offset = offset + 3600;

            String name;
            if (offset >= 0) {
                name = " GMT+";
            }
            else {
                name = " GMT-";
                offset = -offset;
            }

            result = result + name + intToStr(offset / 3600, 2) + intToStr(offset % 3600, 2);
        }

        return result;
    }

    public static String[] getStringElements(String list, char separator){

        Vector temp = new Vector();

        list = list + separator;
        int len = list.length();
        int p = 0;

        while (p < len) {
            int q = p;
            boolean quote = false;

            while ((q < len) && ((list.charAt(q) != separator) || quote)) {
                if (list.charAt(q) == '"') quote = !quote;
                q++;
            }

            String element = list.substring(p, q).trim();
            if (element.length() != 0) {
                temp.addElement(element);
            }
            p = q + 1;
        }

        String[] result = new String[temp.size()];
        for (int i = 0; i < temp.size(); i++) {
            result[i] = (String)(temp.elementAt(i));
        }

        return result;
    }

    public static String getStringName(String s) {
        int p = s.indexOf(':');

        if (p == -1) {
            return null;
        }
        else {
            return s.substring(0, p);
        }
    }

    public static String getStringValue(String s) {
        String value;

        int p = s.indexOf(':');

        if (p == -1) {
            value = s;
        }
        else {
            value = s.substring(p + 1);
        }

        value = value.trim();

        if ((value.length() > 1) && (value.charAt(0) == '"') && ((value.charAt(value.length() - 1) == '"'))) {
            value = value.substring(1, value.length() - 1);
        }

        return value;
    }

    public static String getRandomString() {
        return Long.toString(System.currentTimeMillis(), 36) + "." +
        Integer.toString(Math.abs(new Random().nextInt()), 36);
    }

    Vector getLines() {
        return lines;
    }
}
