package Mail;

import java.io.ByteArrayOutputStream;
import java.util.Vector;

import lib.Utils.Utils;

public class MimeDecoder {
    private Vector lines;

    private int begin;

    private int end;

    private String type;

    private String name;

    private String encoding;

    private String charset;

    private Vector parts;

    public MimeDecoder(Message message) {
        init(message.getLines(), 0, message.getLines().size());
    }

    private MimeDecoder(MimeDecoder parent, int begin, int end) {
        init(parent.lines, begin, end);
    }

    private void init(Vector lines, int begin, int end) {
        this.lines = lines;

        String s = (String)lines.elementAt(begin);
        while (!s.equals("")) {
            begin++;

            while (begin < end) {
                String t = (String)lines.elementAt(begin);
                if (t.startsWith(" ") || t.startsWith("\t")) {
                    s = s + t;
                    begin++;
                }
                else break;
            }

            String name = Message.getStringName(s).toLowerCase();

            if (name.equals("content-type")) {
                String[] elements = Message.getStringElements(Message.getStringValue(s), ';');

                type = elements[0].toLowerCase();
                
                if(elements.length>1)
                	if(elements[1].indexOf("charset=")>-1){
                		charset = elements[1].toLowerCase();
                		charset = Utils.replace(charset, "charset=", "");
                		charset = Utils.replace(charset, "\"", "");
                }
                
                for (int i = 1; i < elements.length; i++) {
                    if (type.startsWith("multipart/") && getStringName(elements[i]).toLowerCase().equals("boundary")) {
                        String boundary = "--" + getStringValue(elements[i]);

                        parts = new Vector();
                        for (int j = begin; j < end; j++) {
                            if (getLine(j).startsWith(boundary)) {
                                parts.addElement(new Integer(j));
                            }
                        }
                    }

                    else if (getStringName(elements[i]).toLowerCase().equals("name")) {
                        this.name = getStringValue(elements[i]);
                    }
                }
            }

            else if (name.equals("content-transfer-encoding")) {
                encoding = Message.getStringValue(s);
            }

            if (begin < end) {
                s = (String)lines.elementAt(begin);
            }
            else {
                s = "";
            }
        }

        this.begin = begin;
        this.end = end;
    }

    private String getLine(int index) {
        return (String)lines.elementAt(index);
    }

    public int getBodyLineCount() {
        return end - begin;
    }

    public String getBodyLine(int index) throws ArrayIndexOutOfBoundsException {
        if ((index < 0) || (index >= end - begin)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        return (String)lines.elementAt(begin + index);
    }

    public byte[] getBodyBytes() {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        for (int i = 0; i < getBodyLineCount(); i++) {
            decode(getBodyLine(i), bos);
        }

        return bos.toByteArray();
    }

    public int getPartCount() {
        if (parts == null) {
            return 0;
        }
        else {
            return parts.size() - 1;
        }
    }

    public MimeDecoder getPart(int index) {
        if ((index < 0) || (index >= getPartCount())) {
            throw new ArrayIndexOutOfBoundsException(index);
        }

        int i = ((Integer)parts.elementAt(index)).intValue();
        int j = ((Integer)parts.elementAt(index + 1)).intValue();

        return new MimeDecoder(this, i + 1, j);
    }

    public String getType() {
        return type;
    }

    public String getName() {
        return name;
    }

    public String getEncoding() {
        return encoding;
    }

    public String getCharset() {
        return charset;
    }

    public static String getStringName(String s) {
        int p = s.indexOf('=');

        if (p == -1) {
            return null;
        }
        else {
            return s.substring(0, p);
        }
    }

    public static String getStringValue(String s) {
        String value;

        int p = s.indexOf('=');

        if (p == -1) {
            value = s;
        }
        else {
            value = s.substring(p + 1);
        }

        value = value.trim();

        if ((value.length() > 1) && (value.charAt(0) == '"') && ((value.charAt(value.length() - 1) == '"'))) {
            value = value.substring(1, value.length() - 1);
        }

        return value;
    }

    private static int decode(char c) {
        if (c >= 'A' && c <= 'Z')
            return ((int) c) - 65;
        else if (c >= 'a' && c <= 'z')
            return ((int) c) - 97 + 26;
        else if (c >= '0' && c <= '9')
            return ((int) c) - 48 + 26 + 26;
        else switch (c) {
            case '+': return 62;
            case '/': return 63;
            case '=': return 0;
            default:  throw new RuntimeException("Illegal MIME character '" + c + "'");
        }
    }

    private static void decode(String s, ByteArrayOutputStream bos) {
        int i = 0;
        int len = s.length();

        while (true) {
            while (i < len && s.charAt(i) <= ' ') i++;

            if (i == len) break;

            int tri = (decode(s.charAt(i)) << 18)
                + (decode(s.charAt(i+1)) << 12)
                + (decode(s.charAt(i+2)) << 6)
                + (decode(s.charAt(i+3)));

            bos.write((tri >> 16) & 255);
            if (s.charAt(i+2) == '=') break;
            bos.write((tri >> 8) & 255);
            if (s.charAt(i+3) == '=') break;
            bos.write(tri & 255);

            i += 4;
        }
    }
}
