package Mail;

import java.io.IOException;

import lib.Utils.Utils;

public class Pop3Client extends InboxClient {

    public Pop3Client(){
        super(null);
    }

    public Pop3Client(Connection connection){
        super(connection);
    }

    public void open(String host, int port, String user, String pass, boolean ssl) throws IOException, MailException {
        if (connected()) close();

        connection.open(host, (port == 0 ? 110 : port), ssl);

        try {
            execute(null);
            execute("USER " + user);
            execute("PASS " + pass);
        }
        catch (MailException e) {
            connection.close();

            throw e;
        }
    }

    public void close() throws IOException, MailException {
        if (connected()) {
            execute("QUIT");
            connection.close();
        }
    }

    private String execute(String command) throws IOException, MailException {
        if (command != null) connection.send(command);

        String result = connection.receive();

        if ((result.length() > 1) && (result.charAt(0) == '-')) {
            throw new MailException(result);
        }

        return result;
    }

    public int getMessageCount() throws IOException, MailException {
        String buffer = execute("STAT");

        int space1 = buffer.indexOf(' ');
        int space2 = buffer.indexOf(' ', space1 + 1);

        return Integer.parseInt(buffer.substring(space1 + 1, space2));
    }

    public long getMessagesSize() throws IOException, MailException {
        String buffer = execute("STAT");

        int space1 = buffer.indexOf(' ');
        int space2 = buffer.indexOf(' ', space1 + 1);

        return Long.parseLong(buffer.substring(space2+1));
    }

    public String getLast() throws IOException, MailException {
        return execute("LAST");
    }

    private Message receiveMessage(Message message) throws IOException, MailException {
        String line = "";

        String buffer = connection.receive();
        while (!(buffer.equals(""))) {
            if ((buffer.length() > 1) && (buffer.charAt(0) == '.')) {
                buffer = buffer.substring(1);
            }

            if (buffer.startsWith(" ") || buffer.startsWith("\t")) {
                line = line + "\r\n" + buffer;
            }
            else {
                if ((line.length() != 0) /* && isGoodHeader(line) */) message.addHeaderLine(line);
                line = buffer;
            }

            buffer = connection.receive();
        }

        if ((line.length() != 0) /* && isGoodHeader(line) */) message.addHeaderLine(line);

        buffer = connection.receive();
        while (!(buffer.equals("."))) {
            if ((buffer.length() > 1) && (buffer.charAt(0) == '.')) {
                buffer = buffer.substring(1);
            }

            message.addBodyLine(buffer);
            buffer = connection.receive();
        }

        return message;
    }

    public Message getMessage(int index, int maxLines) throws IOException, MailException {
    	Message message = new Message();
        message.size = Utils.getKBytes(getSize(index))+" ";
    	
        if (maxLines >= 0) {
            execute("TOP " + (index + 1) + " " + maxLines);
        }
        else {
            execute("RETR " + (index + 1));
        }
        return receiveMessage(message);
    }

    public Message getMessage(int index) throws IOException, MailException {
        return getMessage(index, -1);
    }

    public Message getHeaders(int index) throws IOException, MailException {
        return getMessage(index, 0);
    }

    public void removeMessage(int index) throws IOException, MailException {
        execute("DELE " + (index + 1));
    }

    public String getUniqueId(int index) throws IOException, MailException {
        String buffer = execute("UIDL " + (index + 1));

        int space = buffer.indexOf(' ', 4);

        return buffer.substring(space + 1);
    }

    public int getSize(int index) throws IOException, MailException {
        String buffer = execute("LIST " + (index + 1));

        int space = buffer.indexOf(' ', 4);

        return Integer.parseInt(buffer.substring(space + 1));
    }
    
    //==========  ==============
    public int getBytesIn(){
    	return connection.bytesIn;
    }
    public int getBytesOut(){
    	return connection.bytesOut;
    }
}
