package Mail;

import java.io.IOException;
import java.util.Vector;

public class SmtpClient {

    private Connection connection;

    public SmtpClient(Connection connection) {
        if (connection == null) connection = new Connection();
 
        this.connection = connection;
    }

    public void open(String host, int port, String user, String pass, boolean ssl) throws IOException, MailException {
        if (connected()) close();

        connection.open(host, (port == 0 ? 25 : port), ssl);

        try {
            execute(null);

            if ((user != null) && (pass != null)) {
                execute("EHLO " + "InetTools_EMail");
                execute("AUTH PLAIN " + getAuthString(user, pass));
            }else {
                execute("HELO " + "InetTools_EMail");
            }
        }
        catch (MailException e) {
            connection.close();
            throw e;
        }
    }

    public void close() throws IOException, MailException {
        if (connected()) {
            execute("QUIT");
            connection.close();
        }
    }

    public boolean connected() {
        return connection.connected();
    }

    private String execute(String command) throws IOException, MailException {
        if (command != null) connection.send(command);

        String line = connection.receive();
        String result = line;
        while ((line.length() >= 4) && (line.charAt(3) == '-')) {
            line = connection.receive();
            result = result + line.substring(3);
        }

        char c = result.charAt(0);
        if ((c == '4') || (c == '5')) {
            throw new MailException(result);
        }

        return result;
    }

    public void sendMessage(Message message) throws IOException, MailException {
        sendMessage(new Envelope(message, true));
    }

    public void sendMessage(Envelope envelope) throws IOException, MailException {
        execute("MAIL FROM: <" + Message.getMachineAddress(envelope.getSender()) + ">");

        for (int i = 0; i < envelope.getRecipientCount(); i++) {
            String[] list = Message.getStringElements(envelope.getRecipient(i), ',');

            for (int j = 0; j < list.length; j++) {
                execute("RCPT TO: <" + Message.getMachineAddress(list[j]) + ">");
            }
        }

        execute("DATA");
        Vector lines = envelope.getMessage().getLines();
        for (int i = 0; i < lines.size(); i++) {
            String s = (String)lines.elementAt(i);

            if ((s.length() != 0) && (s.charAt(0) == '.')) {
                s = '.' + s;
            }

            connection.send(s);
        }

        execute(".");
    }

    public boolean getDebug() {
        return connection.getDebug();
    }

    public void setDebug(boolean value) {
        connection.setDebug(value);
    }

    private static String getAuthString(String user, String pass) {

        char[] charTab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".toCharArray();

        byte[] data = ("\000" + user + "\000" + pass).getBytes();

        StringBuffer buf = new StringBuffer();

        int start = 0;
        int len = data.length;

        int end = len - 3;
        int i = start;

        while (i <= end) {
            int d = ((((int) data [i]) & 0x0ff) << 16)
                | ((((int) data [i+1]) & 0x0ff) << 8)
                | (((int) data [i+2]) & 0x0ff);

            buf.append(charTab [(d >> 18) & 63]);
            buf.append(charTab [(d >> 12) & 63]);
            buf.append(charTab [(d >> 6) & 63]);
            buf.append(charTab [d & 63]);

            i += 3;
        }

        if (i == start + len - 2) {
            int d = ((((int) data [i]) & 0x0ff) << 16)
                | ((((int) data [i+1]) & 255) << 8);

            buf.append(charTab [(d >> 18) & 63]);
            buf.append(charTab [(d >> 12) & 63]);
            buf.append(charTab [(d >> 6) & 63]);
            buf.append("=");
        }
        else if (i == start + len - 1) {
            int d = (((int) data [i]) & 0x0ff) << 16;

            buf.append(charTab [(d >> 18) & 63]);
            buf.append(charTab [(d >> 12) & 63]);
            buf.append("==");
        }

        return buf.toString();
    }
    //==========  ==============
    public int getBytesIn(){
    	return connection.bytesIn;
    }
    public int getBytesOut(){
    	return connection.bytesOut;
    }
}
